@@ -45,6 +45,8 @@ BEGIN {
       own_cabal_prerequisites
       hashed_id_to_virtual_installable
       config_to_package_id
+      config_to_package_name_version
+      is_package_exposed
       init_hs_env
       clean_recipe
       make_setup_recipe
@@ -412,11 +414,31 @@ sub own_cabal_prerequisites {
         my $pkg_config = $2;
 
         if (-d $pkg_config) {
-            # https://downloads.haskell.org/cabal/Cabal-3.0.0.0/doc/users-guide/installing-packages.html#cmdoption-setup-register-gen-pkg-config
-            # If the package registration is a directory, choose the first one since the other(s) will be internal libraries that we don't want to install.
+            # If the package registration is a directory, load all .conf files
             my @pkg_configs = glob("$pkg_config/*");
-            $pkg_config = "$pkg_config/$pkg_config";
-            run('mv', $pkg_configs[0], $pkg_config);
+            my $ghc_pkg = ghc_pkg_command();
+            load_ghc_database($ghc_pkg, $tmp_db, @pkg_configs);
+
+            # Gather all the depends from all the libraries
+            my $locals
+                = run_quiet($ghc_pkg, '--package-db', $tmp_db,
+                            '--simple-output', '--show-unit-ids', 'list');
+            my @local_pkgs = uniq(split($SPACE, $locals // $EMPTY));
+            for my $pkg (@local_pkgs) {
+                my $depends = run($ghc_pkg, '--package-db', $tmp_db, 'field',
+                                  '--unit-id', '--simple-output', $pkg,
+                                  'depends');
+                push(@hashed_ids, split($SPACE, $depends // $EMPTY));
+            }
+
+            # Filter out those depends that are internal
+            @hashed_ids = uniq(@hashed_ids);
+            my %local_hash = map { $_ => 1 } @local_pkgs;
+            @hashed_ids = grep { !$local_hash{$_} } @hashed_ids;
+
+            run(qw{rm -rf}, $pkg_config);
+
+            return @hashed_ids;
         }
 
         my $ghc_pkg = ghc_pkg_command();
@@ -523,6 +545,44 @@ sub config_to_package_id {
     return $package_id;
 }
 
+=item config_to_package_name_version
+
+=cut
+
+sub config_to_package_name_version {
+    my ($config) = @_;
+
+    my $cabal_contents = path($config)->slurp_utf8;
+
+    die encode_utf8('Cannot get package name from config file')
+        unless $cabal_contents =~ /^ name \s* : \s* (\S*) \s* $/imx;
+
+    my $package_name = $1;
+
+    die encode_utf8('Cannot get package version from config file')
+        unless $cabal_contents =~ /^ version \s* : \s* (\S*) \s* $/imx;
+
+    my $package_version = $1;
+
+    return ($package_name, $package_version);
+}
+
+=item is_pkg_internal
+
+=cut
+
+sub is_package_exposed {
+    my ($package_id, $ghc_pkg, $package_db) = @_;
+
+    my $exposed
+        = run_quiet($ghc_pkg, '--package-db', $package_db, 'field',
+                    '--unit-id', '--simple-output', $package_id, 'exposed');
+    return 1
+        if $exposed eq 'True';
+
+    return 0;
+}
+
 =item init_hs_env
 
 =cut
